/* 2- Inicialice un tablero para el juego la batalla naval. El tablero es una matriz bidimensional, donde cada casillero es un puntero a registro. Para cada celda de la matriz, la memoria se toma dinmicamente. Este problema es el correspondiente al nmero 5 del captulo 6 (es aconsejable tratar de identificar las diferencias).*/
/* Solucin en lenguaje C */

#include <stdio.h>
#include <stdlib.h>

#define COLS 20
#define FILAS 10

typedef enum {ACORAZADO, CRUCERO, PORTAAVIONES} t_tipo;
typedef enum {FALSE, TRUE} t_bool;

typedef struct {
	t_bool ocupado;
	t_tipo tipo;
	t_bool hundido;
	} t_casillero;

/* 't_tablero' es una matriz de punteros a 't_casillero' */
typedef t_casillero* t_tablero[FILAS][COLS];

/* Prototipos */
void inicializar_tablero(t_tablero);
void mostrar_tablero(t_tablero);
void liberar_tablero(t_tablero);

int main()
{
	t_tablero tablero;
	inicializar_tablero(tablero);
	mostrar_tablero(tablero);
	liberar_tablero(tablero);

	return 0;
}

void inicializar_tablero(t_tablero tablero)
{
	int i,j;

	for (i=0; i<FILAS; i++)
	for (j=0; j<COLS; j++)
	{

	/* Se toma memoria dinmicamente para cada celda */
	tablero[i][j] = malloc( sizeof(t_casillero) );
	if (tablero[i][j] == NULL)
	{
		/* No se pudo tomar memoria dinmica. Se liberan las */
		/* posiciones que pudiesen haber sido asignadas hasta */
		/* este punto y se sale de la aplicacin. */
		liberar_tablero(tablero);
		fprintf(stderr, "Error intentando tomar memoria dinmica.");
		exit(1);
		}
		/* Equivalente a: (*tablero[i][j]).ocupado = FALSE; */
		tablero[i][j]->ocupado = FALSE;
	}

/* 1 acorazado */
tablero[4][18]->ocupado = TRUE;
tablero[4][18]->tipo = ACORAZADO;
tablero[4][18]->hundido = FALSE;
tablero[5][18]->ocupado = TRUE;
tablero[5][18]->tipo = ACORAZADO;
tablero[5][18]->hundido = FALSE;
tablero[6][18]->ocupado = TRUE;
tablero[6][18]->tipo = ACORAZADO;
tablero[6][18]->hundido = FALSE;

/* 2 cruceros */
tablero[9][2]->ocupado = TRUE;
tablero[9][2]->tipo = CRUCERO;
tablero[9][2]->hundido = FALSE;
tablero[9][3]->ocupado = TRUE;
tablero[9][3]->tipo = CRUCERO;
tablero[9][3]->hundido = FALSE;
tablero[3][9]->ocupado = TRUE;
tablero[3][9]->tipo = CRUCERO;
tablero[3][9]->hundido = FALSE;
tablero[4][9]->ocupado = TRUE;
tablero[4][9]->tipo = CRUCERO;
tablero[4][9]->hundido = FALSE;

/* 3 portaaviones */
tablero[1][1]->ocupado = TRUE;
tablero[1][1]->tipo = PORTAAVIONES;
tablero[1][1]->hundido = FALSE;
tablero[6][12]->ocupado = TRUE;
tablero[6][12]->tipo = PORTAAVIONES;
tablero[6][12]->hundido = FALSE;
tablero[8][19]->ocupado = TRUE;
tablero[8][19]->tipo = PORTAAVIONES;
tablero[8][19]->hundido = FALSE;
}

void mostrar_tablero(t_tablero tablero)
{
int i,j;
printf("\t");
for (i=0; i<COLS+2; i++)
	printf("-");
printf("\n");

for (i=0; i<FILAS; i++)
{
	printf("\t|");
	for (j=0; j<COLS; j++)
	{
	if (!tablero[i][j]->ocupado) /* o: (*tablero[i][j]).ocupado */
		printf(" ");
	else
		switch ( (*tablero[i][j]).tipo ) /* o: tablero[i][j]->tipo */
		{
		case ACORAZADO: printf("#");
		break;
		case CRUCERO: printf("X");
		break;
		case PORTAAVIONES: printf("@");
		break;
		}
	}
	printf("|\n");
}

printf("\t");
for (i=0; i<COLS+2; i++)
	printf("-");
printf("\n");
}

void liberar_tablero(t_tablero tablero)
{
	int i,j;
	for (i=0; i<FILAS; i++)
		for (j=0; j<COLS; j++)
			if (tablero[i][j] != NULL) /* Tiene memoria asignada. */
			free( tablero[i][j] );
}